package de.hsrm.cs.javaee8.statelessmaven.web.test;

import java.io.File;
import java.net.URL;

import javax.inject.Inject;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.drone.api.annotation.Drone;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.test.api.ArquillianResource;
import org.jboss.arquillian.warp.Activity;
import org.jboss.arquillian.warp.Inspection;
import org.jboss.arquillian.warp.Warp;
import org.jboss.arquillian.warp.WarpTest;
import org.jboss.arquillian.warp.jsf.AfterPhase;
import org.jboss.arquillian.warp.jsf.BeforePhase;
import org.jboss.arquillian.warp.jsf.Phase;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.StringAsset;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.descriptor.api.Descriptors;
import org.jboss.shrinkwrap.descriptor.api.application7.ApplicationDescriptor;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.hsrm.cs.javaee8.statelessmaven.web.GeometricModelHandler;

/**Integration test for JSF page and JSF backing bean: fills a JSF form, submit it and checks the results.
 * 
 * @author Wolfgang Knauf
 *
 */
@WarpTest
@RunWith(Arquillian.class)
@RunAsClient
public class WarpIT
{

  /**
   * Liefert das EAR-Deployment, das auf den Server gepackt werden soll.
   * 
   * @return EAR-Archiv
   */
  @Deployment
  public static Archive<?> getEarArchive()
  {
    EnterpriseArchive ear = ShrinkWrap.create(EnterpriseArchive.class, "StatelessMaven-ear.ear");

    // Wir stehen aktuell im Verzeichnis des "StatelessMaven-web"-Projekts. Also
    // eine Ebene höher gehen und in das "StatelessMaven-ejb"-Projekt gehen.

    File f = new File("../StatelessMaven-ejb/target/StatelessMaven-ejb.jar");
    System.err.println("Exists: " + f.getAbsolutePath() + "=>" + f.exists());
    JavaArchive ejbJar = ShrinkWrap.create(ZipImporter.class, "StatelessMaven-ejb.jar").importFrom(f)
        .as(JavaArchive.class);
    ear.addAsModule(ejbJar);

    f = new File("../StatelessMaven-web/target/StatelessMaven-web.war");
    if (f.exists() == false)
    {
      System.err.println("File " + f.getAbsolutePath() + " does not exist.");
      // Exception-Message geht leider verloren...
      throw new RuntimeException("File " + f.getAbsolutePath() + " does not exist.");
    }
    System.err.println("Exists: " + f.getAbsolutePath() + "=>" + f.exists());
    WebArchive war = ShrinkWrap.create(ZipImporter.class, "StatelessMaven-web.war").importFrom(f).as(WebArchive.class);
    ear.addAsModule(war);

    // "application.xml" händisch aufbauen mit Hilfe des "Shrinkwrap
    // Descriptors"-Projekts:
    ApplicationDescriptor descriptor = Descriptors.create(ApplicationDescriptor.class);

    descriptor.applicationName("StatelessMaven-ear");
    // Webmodul bauen:
    descriptor.createModule().getOrCreateWeb().contextRoot("/StatelessMaven-web").webUri("StatelessMaven-web.war");

    // EJB-Modul reinpacken:
    descriptor.createModule().ejb("StatelessMaven-ejb.jar");

    // In EAR-Datei schreiben:
    ear.setApplicationXML(new StringAsset(descriptor.exportAsString()));

    // Manifest-Datei haben wir nicht.

    // Packages mit Test-Klassen zum Deployment zufügen:
    war.addPackage("de.hsrm.cs.javaee8.statelessmaven.web.test");

    // EAR exportieren zum Anschauen:
    // ear.as(ZipExporter.class).exportTo(new File("c:\\temp\\test.ear"), true);

    return ear;
  }

  @Drone
  private WebDriver browser;

  /**
   * The request URL (root of the web app) is injected)
   */
  @ArquillianResource()
  private URL deploymentUrl;

  /**This is the test method: fills a JSF form, submit it and checks the results.
   * 
   * @throws Exception Any exception...
   */
  @Test
  public final void browserTest() throws Exception
  {
    System.out.println("Testing client website ... " + deploymentUrl);

    Warp.initiate(new Activity()
    {
      @Override
      public void perform()
      {

        System.out.println("Activity.perform - initial page");
        browser.navigate().to(deploymentUrl.toExternalForm() + "geometricmodel.xhtml");
      }
    })
    .inspect(new Inspection()
    {
      private static final long serialVersionUID = 1L;

      // Nothing to be done...

    });

    System.out.println("Done initial request");

    Warp.initiate(new Activity()
    {
      public void perform()
      {
        System.out.println("Activity.perform - filling and submitting form");

        WebElement txt = browser.findElement(By.id("formGeometricModelInput:a"));
        // Delete old value, then write new values to the form:
        txt.clear();
        txt.sendKeys("1");

        txt = browser.findElement(By.id("formGeometricModelInput:b"));
        txt.clear();
        txt.sendKeys("2");

        txt = browser.findElement(By.id("formGeometricModelInput:c"));
        txt.clear();
        txt.sendKeys("3");

        // Submit the form:
        WebElement btn = browser.findElement(By.id("formGeometricModelInput:calculate"));
        btn.click();

        WebElement result = browser.findElement(By.id("formGeometricModelInput:volume"));
        System.out.println("Activity.perform: volume in client page is: " + result.getText());
        Assert.assertEquals("6.0", result.getText());
        
        result = browser.findElement(By.id("formGeometricModelInput:surface"));
        System.out.println("Activity.perform: surface in client page is: " + result.getText());
        Assert.assertEquals("22.0", result.getText());

        System.out.println("Activity.perform DONE!!!");
      }
    })
        //.observe(HttpFilters.request().uri().contains("geometricmodel.faces"))
        .inspect(new Inspection()
        {
          private static final long serialVersionUID = 1L;

          @Inject
          GeometricModelHandler hmb;

          @BeforePhase(Phase.RENDER_RESPONSE)
          public void beforeRenderResponse()
          {
            System.out.println("beforeRenderResponse: " + hmb.toString());
          }

          @AfterPhase(Phase.RENDER_RESPONSE)
          public void afterRenderResponse()
          {
            System.out.println("afterRenderResponse: " + hmb.toString());
          }

          @AfterPhase(Phase.UPDATE_MODEL_VALUES)
          public void afterUpdateModelValues()
          {
            System.out.println("afterUpdateModelValues: " + hmb.toString());
          }

          @BeforePhase(Phase.UPDATE_MODEL_VALUES)
          public void beforeUpdateModelValues()
          {
            System.out.println("beforeUpdateModelValues: " + hmb.toString());
          }

          @BeforePhase(Phase.RESTORE_VIEW)
          public void beforeRestoreView()
          {
            System.out.println("beforeRestoreView: " + hmb.toString());
          }

          @AfterPhase(Phase.RESTORE_VIEW)
          public void afterRestoreView()
          {
            System.out.println("afterRestoreView: " + hmb.toString());
          }

          @BeforePhase(Phase.APPLY_REQUEST_VALUES)
          public void beforeApplyRequestValues()
          {
            System.out.println("beforeApplyRequestValues: " + hmb.toString());
          }

          @AfterPhase(Phase.APPLY_REQUEST_VALUES)
          public void afterApplyRequestValues()
          {
            System.out.println("afterApplyRequestValues:" + hmb.toString());
          }

          @BeforePhase(Phase.PROCESS_VALIDATIONS)
          public void beforeProcessValidations()
          {
            System.out.println("beforeProcessValidations:" + hmb.toString());
          }

          @AfterPhase(Phase.PROCESS_VALIDATIONS)
          public void afterProcessValidations()
          {
            System.out.println("afterProcessValidations:" + hmb.toString());
          }

          @BeforePhase(Phase.INVOKE_APPLICATION)
          public void beforeInvokeApplication()
          {
            System.out.println("beforeInvokeApplication:" + hmb.toString());
          }

          @AfterPhase(Phase.INVOKE_APPLICATION)
          public void afterInvokeApplication()
          {
            System.out.println("afterInvokeApplication: " + hmb.toString());
            Assert.assertEquals("invalid volume", 6, hmb.getVolume(), 0.0);
            Assert.assertEquals("invalid surface", 22, hmb.getSurface(), 0.0);
          }
        });

  }
}
